<?PHP

// ---------------------------------------------------------------
//
// -----------------------------------------------------
// copyright 2000 by voxeo corporation. (see LGPL.txt)
// -----------------------------------------------------
//
// v1.0 Coded in PHP
// v1.0 Coded by Stephen J. Lewis
//
// This application is a simple "Telephone Directory" or contact list
// that uses a comma delimited file for contact information.  It allows
// the caller to dial into a number, and then place an outbound call
// to a phone number in the list or send a voice message as an email
// attachment to an email address in the list.  If either of those are
// missing, the caller can still use the other function.  This application
// was also coded in Perl and ASP.
//
// This zip file for this application contains the following files:				 
//     contactlist.php       <---- This file
//     contactlist.asp       <---- ASP version
//     contactlist.pl        <---- Perl version
//     contactlist.ini       <---- Comma delimited configuration file.  See the
//                                 header in that file for more details.
//     LGPL.txt              <---- opensource license agreement
//     readme.txt            <---- Introductory text file
//
// Several audio files are associated with this application:
//           greeting.wav
//           nocontactinfo.wav
//           onlyphone.wav
//           onlyemail.wav
//           bothphoneemail.wav
//           starkey.wav
//           returningtomainmenu.wav
//           emailmessage.wav
//           errormessage.wav
//           inifileerror.wav
//           messagesent.wav
//           holdmusic.wav
//           callfailure.wav
//
// All audio files are PCM format, 8-bit, mono, unsigned.
//
//----------------------------------------------------------------


//------------------------------------------
// Change this variable to reflect your path
//------------------------------------------

$Filename = "contactlist.ini";

//-------------------------------------------------------------------------------------------

function Initialize() {
  echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"; 
  echo "<callxml>";
  
  //------------------------------------------------------------------------
  // This annoying little variable is simply incremented and passed along to
  // avoid the AGS's caching.  This helps to ensure that a different URL
  // query string will be passed.
  //------------------------------------------------------------------------

  if ($HTTP_GET_VARS["cachekiller"] == "") {
    echo "<assign var=\"cachekiller\" value=\"1\"/>";
  }  // if statement
  else {
    $HTTP_GET_VARS["cachekiller"]++;
	echo "<assign var=\"cachekiller\" value=\"" . $HTTP_GET_VARS["cachekiller"] . "\"/>";
  }  // else statement
}  // Initialize function

//-------------------------------------------------------------------------------------------

function Closure() {
  echo "</callxml>";
}  // Closure function

//-------------------------------------------------------------------------------------------

function GetContactList() {
  global $Filename;
  
  $ContactInfo = Array();
  
  if ( file_exists($Filename) ) {
   //--------------------------------------------
   // Grab the file and start reading in the data
   //--------------------------------------------
  
   $ContactListFile = fopen($Filename, "r+"); 
   $Loopy           = 0;
   
   while ( !feof($ContactListFile) ) {
     //---------------------------------------------------------------
	 // Read in a line of up to 4k in size.  This is just allocating a
	 // a large chunk that is ultimately unneccessary as the PHP fgets
	 // line should stop at the newline/carriage return.
	 //---------------------------------------------------------------
	 
     $Line = fgets($ContactListFile, 4096); 
	 
	 //---------------------------------------------------
	 // We only want lines that do not start with a # sign
	 // (which is a commented line), and lines that have
	 // commas in them.
	 //---------------------------------------------------
	 
	 $IsThereAComma = strpos($Line, ",");
	 
	 if (($Line[0] != "#") && ( $IsThereAComma )) {
	   //-----------------------------------------------
	   // Increase our loop counter and add to our array
	   //-----------------------------------------------

	   $ContactInfo[$Loopy] = $Line;
	   $Loopy++;	   
	 }  // if statement
   }  // while statement
   
   //---------------
   // Close our file
   //---------------
   
    fclose($ContactListFile);  
  }  // if statement (file exists)
  else {
    //--------------------------------------------
    // Oops, no configuration file found.  Without
	// a config file, there can be no contact list.  
	//---------------------------------------------
    
    $ContactInfo[0] = "FILE NOT FOUND";
  }  // else statement (file does not exist)
  
  //-----------------
  // Return our array
  //-----------------
  
  return $ContactInfo;
}  // GetContactList function

//-------------------------------------------------------------------------------------------

function GetTextMessage($ContactList) {
  global $HTTP_GET_VARS;
  
  $Loopy      = 0;
  $Text       = Array();
  $TempArray  = Array();
  
  //---------------------------------------------------
  // We want to use the offset variable because we only
  // read 10 contacts at a time.  We obviously loop no
  // more than 10 times as a result of that fact.
  //--------------------------------------------------- 
   
  while (($ContactList[$Loopy + $HTTP_GET_VARS["offset"]] != "") && ($Loopy < 10)) {
    //-----------------------------------------------------------------------------
    // Here we are simply splitting the line into its component parts via the comma
	//-----------------------------------------------------------------------------
  
    $TempArray = explode( ",", $ContactList[$Loopy + $HTTP_GET_VARS["offset"]]);
						
	//---------------------------------------------------------------------
	// If there is no name, then simply use "Unnamed Contact" when reading
	//---------------------------------------------------------------------					
						
    if ($TempArray[0] == "") { $TempArray[0] == "Unnamed Contact"; }
	
	$Text["Message"]    = $Text["Message"] . "For " . $TempArray[0] . " press " . $Loopy . ",";
	$Text["Names"]      = $Text["Names"]  . $TempArray[0] . ",";
	$Text["Phones"]     = $Text["Phones"] . $TempArray[1] . ",";
	$Text["Emails"]     = $Text["Emails"] . $TempArray[2] . ",";
	$Text["TermDigits"] = $Text["TermDigits"] . $Loopy;
	$Loopy++;				
  }  // while statement
  
  //-----------------------------------------------------------
  // If we are in the middle of the contact list, then tack
  // on a * as termdigit as the "move to the previous page" key.
  //------------------------------------------------------------
  
  if ($HTTP_GET_VARS["offset"] != "0") { 
    $Text["TermDigits"] = $Text["TermDigits"] . "*"; 
	$Text["Message"]    = $Text["Message"] . 
	                            "To move back in your list, press the star key,";			
	
  }
  
  //----------------------------------------------------------
  // If next item in the contact list is NOT empty, then tack
  // on a # as termdigit as the "move to the next page" key.
  //----------------------------------------------------------  
  
  if (($ContactList[$Loopy + $HTTP_GET_VARS["offset"]]) != "") {
    $Text["TermDigits"] = $Text["TermDigits"] . "#";
	$Text["Message"]    = $Text["Message"] . 
	                            "To move forward in your list, press the pound key,";			
  }  // if statement
  
  return $Text;
}  // GetTextMessage function

//-------------------------------------------------------------------------------------------

function GetPiece($TempString, $WhichOne) {
  $TempArray = Array();
  $TempArray = explode(",", $TempString);

  return trim($TempArray[$WhichOne]);
}  // GetPiece function

//-------------------------------------------------------------------------------------------

function MainMenu() {
  global $HTTP_GET_VARS;

  $ContactList = Array();
  $ContactList = GetContactList();
  
  
  if ($ContactList[0] == "FILE NOT FOUND") {
    echo "<playaudio value=\"inifileerror.wav\"/>";
  }  // if statement
  else {
    //------------------------------------------------------------
    // Because we only play 10 contacts at a time, we only want to
    // play the initial greeting file if we are calling into the
    // list for the first time.
    //------------------------------------------------------------
  
    if ($HTTP_GET_VARS["offset"] == "") { 
	  $HTTP_GET_VARS["offset"]    = "0"; 
	  $HTTP_GET_VARS["firstpass"] = "YES";
	}  // if statement
  
    if ($HTTP_GET_VARS["firstpass"] == "YES") {
      echo "<playaudio value=\"greeting.wav\"/>";
    }  // if statement
  
    //---------------------------------------------------------------------------
    // Get the text message from the array of contacts (uses the offset variable)
    //---------------------------------------------------------------------------
  
    $Text = GetTextMessage($ContactList);
    
    echo "<block label=\"MainMenu\" " .
                "repeat=\"3\">"; 
			   
    echo "<text termdigits=\"" . $Text["TermDigits"] . "\">";
    echo $Text["Message"];
    echo "  </text>";
  
    //-------------------------------------------------------
    // Now we want to loop through and add our event handlers
    //-------------------------------------------------------
  
    for ($Loopy = 0; $Loopy < strlen($Text["TermDigits"]); $Loopy++) {
      echo "<ontermdigit value=\"" . $Text["TermDigits"][$Loopy] . "\">";
	
	  //---------------------------
	  // Move to previous set of 10
	  //---------------------------
	
	  if ($Text["TermDigits"][$Loopy] == "*") {
	    $Offset = $HTTP_GET_VARS["offset"];
	    $Offset = $Offset - 10;
	  
	    if ($Offset < 0) { $Offset = 0; }
	  
	    echo "<goto value=\"contactlist.php?menu=mainmenu&amp;offset=" . $Offset . "\"/>";	  
	  }  // if statement
	
	  //-----------------------
	  // Move to next set of 10
	  //-----------------------
	
	  elseif ($Text["TermDigits"][$Loopy] == "#") {
	    $Offset = $HTTP_GET_VARS["offset"];
	    $Offset = $Offset + 10;
	  
	    echo "<goto value=\"contactlist.php?menu=mainmenu&amp;offset=" . $Offset . "\"/>";		
	  }  // elseif statement
	
	  //--------------------
	  // Actual contact name
	  //--------------------
	
	  else {
	    $Email = GetPiece($Text["Emails"], $Loopy);
	    $Phone = GetPiece($Text["Phones"], $Loopy);
	    $Name  = GetPiece($Text["Names"],  $Loopy);
	  
	    echo "<goto value=\"contactlist.php?menu=submenu&amp;email=" . $Email . 
	                "&amp;phone="  . $Phone .
				    "&amp;name="   . $Name  .
	  			    "&amp;offset=" . $HTTP_GET_VARS["offset"] . "\" " .
				    "submit=\"*\" method=\"get\"/>";
	  }  // else statement
	
	  echo "</ontermdigit>";
    }  // for loop  
  
    echo "<wait value=\"3s\" termdigits=\"" . $Text["TermDigits"] . "\"/>";
  
    echo "</block>";
  }  // else statement
}  // MainMenu function

//-------------------------------------------------------------------------------------------

function SubMenu() {
  global $HTTP_GET_VARS;
  
  $Email  = $HTTP_GET_VARS["email"];
  $Phone  = $HTTP_GET_VARS["phone"];
  $Name   = $HTTP_GET_VARS["name"];
  $Offset = $HTTP_GET_VARS["offset"];
 
  echo "<block label=\"SubMenu\" repeat=\"3\">";
  
  //--------------------------------------------------------------------
  // We need to play the appropriate audio file depending on the contact
  // information that is stored in the configuration file.
  //--------------------------------------------------------------------
  
  if (($Email == "") && ($Phone == "")) { 
    echo "<playaudio value=\"nocontactinfo.wav\"/>";
	$TermDigits = ""; 
  }  // if statement
  elseif (($Email == "") && ($Phone != "")) { 
    echo "<playaudio value=\"onlyphone.wav\" " . "termdigits=\"1*\"/>"; 
	$TermDigits = "1*";
  }    // elseif statement
  elseif (($Email != "") && ($Phone == "")) { 
    echo "<playaudio value=\"onlyemail.wav\" " . "termdigits=\"2*\"/>"; 
	$TermDigits = "2*";
  }  // elseif statement
  else { 
    echo "<playaudio value=\"bothphoneemail.wav\" " . "termdigits=\"12*\"/>"; 
	$TermDigits = "12*";
  }  // else statement
  
  //-----------------------
  // Read the person's name
  //-----------------------
  
  echo "<text termdigits=\"" . $TermDigits . "\">" . $Name . "</text>";	
  
  //---------------------------------------------------------------------------
  // We do not need to play "Press the Star Key to return to the main menu"
  // if there is no contact information (because we are already returning there.
  //----------------------------------------------------------------------------
  
  if ($TermDigits != "") {
    echo "<playaudio value=\"starkey.wav\" termdigits=\"" . $TermDigits . "\"/>";
  }  // if statement
  
  //--------------------------------------------------------
  // We need to use the appropriate event handlers depending
  // on what contact information we have.
  //--------------------------------------------------------
  
  if ($Phone != "") { 
	echo "<ontermdigit value=\"1\">";
	echo "<goto value=\"contactlist.php?menu=phone&amp;phone=" . $Phone . 
	  			  "&amp;offset=" . $Offset . "\" " .
				  "submit=\"*\" method=\"get\"/>";
	echo "</ontermdigit>";
  }  // if statement (Phone exists)
  
  if ($Email != "") {
	echo "<ontermdigit value=\"2\">";
	echo "<goto value=\"contactlist.php?menu=email&amp;email=" . $Email . 
	  			  "&amp;offset=" . $Offset . "\" " .
				  "submit=\"*\" method=\"get\"/>";	
	echo "</ontermdigit>";		
  }  // if statement (Email exists)
  
  if (($Email != "") || ($Phone != "")) {
	echo "<ontermdigit value=\"*\">";
	echo "<goto value=\"contactlist.php?menu=mainmenu" .
	  			  "&amp;offset=" . $Offset . "\" " .
				  "submit=\"*\" method=\"get\"/>";	
	echo "</ontermdigit>";		  
  }  // if statement (Phone or Email exist)
  else {
	echo "<playaudio value=\"returningtomainmenu.wav\"/>";
	echo "<goto value=\"contactlist.php?menu=mainmenu&amp;offset=" . $Offset . "\"/>";
  }  // else statement  
  
  echo "<wait value=\"3s\" termdigits=\"" . $Text["TermDigits"] . "\"/>";
 
  echo "</block>";
  
  echo "<goto value=\"contactlist.php?menu=mainmenu&amp;offset=" . $Offset . "\"/>";  
}  // SubMenu function

//-------------------------------------------------------------------------------------------

function EmailMenu() {
  global $HTTP_GET_VARS;
  
  echo "<block>";
  echo "<cleardigits/>";
  echo "<playaudio value=\"EmailMessage.wav\" termdigits=\"*#\"/>";
  echo "<recordaudio format=\"audio/wav\" " .
                     "value=\"mailto:" . $HTTP_GET_VARS["email"] . "\" " .
					 "maxtime=\"180s\" " .
					 "maxsilence=\"20s\" " .
					 "cleardigits=\"TRUE\" " .
					 "beep=\"TRUE\" " .
					 "termdigits=\"*#\"/>";
					 
  echo "<onerror>";
  echo "<playaudio value=\"errormessage.wav\"/>";
  echo "<goto value=\"contactlist.php?menu=mainmenu&amp;offset=" . 
  			  $HTTP_GET_VARS["offset"] . "\"/>";   
  echo "</onerror>";
  
  echo "<onmaxtime/>";
  echo "<onmaxsilence/>";
  
  echo "<ontermdigit value=\"*\">";
  echo "<playaudio value=\"messagesent.wav\"/>";  
  echo "<goto value=\"contactlist.php?menu=mainmenu&amp;offset=" . 
  			  $HTTP_GET_VARS["offset"] . "\"/>";   
  echo "</ontermdigit>";
  
  echo "<ontermdigit value=\"#\">";
  echo "<playaudio value=\"messagesent.wav\"/>";  
  echo "<goto value=\"contactlist.php?menu=mainmenu&amp;offset=" . 
  			  $HTTP_GET_VARS["offset"] . "\"/>";   
  echo "</ontermdigit>";  
  
  echo "</block>";
}  // EmailMenu function

//-------------------------------------------------------------------------------------------

function DoTheCall() {
  global $HTTP_GET_VARS;
  
  //----------------------------------------------------
  // This function does the actual outbound call, and
  // then sends back the results via callXML <sendevent>
  // actions.
  //----------------------------------------------------
  
  echo "<block>";

  echo "<call value=\"" . $HTTP_GET_VARS["phone"] . "\" " .
       "maxtime=\"25s\"/>";
		
  //-----------------------------------------------------
  // Send back the appropriate event to the other session
  //-----------------------------------------------------
		
  echo "<onanswer>";
  echo "  <sendevent value=\"success\" session=\"\$parentsessionid;\"/>";
  echo "  <wait value=\"Unlimited\"/>";			   
  echo "</onanswer>";
  
  echo "<oncallfailure>";
  echo "  <sendevent value=\"busy\" session=\"\$parentsessionid;\"/>";
  echo "</oncallfailure>";
  
  echo "<onmaxtime>";
  echo "  <sendevent value=\"timedout\" session=\"\$parentsessionid;\"/>";
  echo "</onmaxtime>";
  
  echo "<onerror>";
  echo "  <sendevent value=\"error\" session=\"\$parentsessionid;\"/>";
  echo "</onerror>";
  
  echo "<onexternalevent value=\"kill\">";
  echo "  <hangup/>";
  echo "</onexternalevent>";
  
  echo "</block>";
}  // DoTheCall function

//----------------------------------------------------------------------------------------


function PhoneMenu() {
  global $HTTP_GET_VARS;
  
  echo "<block>";
	
  //-------------------------------------------------------
  // We need to assign a ParentSessionID variable so that
  // the session we call with the <run> command can send us
  // a callXML event back.  Without assigning that variable,
  // there is no way to remember/determine the parent 
  // session's ID.  Remember, \$ is the escape character for
  // $ in PHP (as seen in the line directly below...)
  //--------------------------------------------------------
	 
  echo "<assign var=\"parentsessionid\" value=\"\$session.id;\"/>";
	
  echo "<run value=\"contactlist.php?menu=dothecall" . 
        "&amp;phone=" . $HTTP_GET_VARS["phone"] . 
		"&amp;offset=" . $HTTP_GET_VARS["offset"] . "\" " .
	    "submit=\"*\" " .
	    "method=\"get\" " .
	    "var=\"ChildSession\"/>";
	   
  echo "<block label=\"Hold Music\" " .
	          "repeat=\"3\">";
	   
  //-----------------------------------------------------------------
  // Play holdmusic while waiting for an event from the other session
  //-----------------------------------------------------------------
		   
  echo "<playaudio value=\"holdmusic.wav\"/>";
	   
  //----------------------------------------------------
  // Conference the call when someone/something picks up
  //----------------------------------------------------
		
  echo "<onexternalevent value=\"success\">";
  echo "  <conference targetsessions=\"\$session.eventsenderID;\"/>";
  echo "  <hangup/>";
  echo "</onexternalevent>";
	 
  //-------------------------------------------
  // Inform the caller when the number was busy
  //-------------------------------------------
		
  echo "<onexternalevent value=\"busy\">";
  echo "  <playaudio value=\"callfailure.wav\"/>";					 
  echo "  <sendevent value=\"kill\" session=\"\$ChildSession;\"/>";
  echo "  <goto value=\"contactlist.php?menu=mainmenu" . 
              "&amp;offset=" . $HTTP_GET_VARS["offset"] . "\" submit=\"*\" method=\"get\"/>";
  echo "</onexternalevent>";
	 
  //-------------------------------------------
  // Inform the caller when the number timedout
  //-------------------------------------------
	
  echo "<onexternalevent value=\"timedout\">";
  echo "  <playaudio value=\"callfailure.wav\"/>";					 
  echo "  <sendevent value=\"kill\" session=\"\$ChildSession;\"/>";
  echo "  <goto value=\"contactlist.php?menu=mainmenu" . 
              "&amp;offset=" . $HTTP_GET_VARS["offset"] . "\" submit=\"*\" method=\"get\"/>";
  echo "</onexternalevent>";
	 
  //-----------------------------------------------
  // Inform the caller when there has been an error
  //-----------------------------------------------  
	 
  echo "<onexternalevent value=\"error\">";
  echo "  <playaudio value=\"callfailure.wav\"/>";					 
  echo "  <sendevent value=\"kill\" session=\"\$ChildSession;\"/>";
  echo "  <goto value=\"contactlist.php?menu=mainmenu" . 
              "&amp;offset=" . $HTTP_GET_VARS["offset"] . "\" submit=\"*\" method=\"get\"/>";
  echo "</onexternalevent>";  
	
  echo "</block>";
	 
  //-------------------------------------------------------
  // In theory, we shouldn't reach this point.  But just in
  // case, we might want to give the user a message.
  //-------------------------------------------------------
	 
  echo "  <playaudio value=\"callfailure.wav\"/>";					 
  echo "  <sendevent value=\"kill\" session=\"\$ChildSession;\"/>";
  echo "  <goto value=\"contactlist.php?menu=mainmenu" . 
              "&amp;offset=" . $HTTP_GET_VARS["offset"] . "\" submit=\"*\" method=\"get\"/>";
  echo "</block>";
}  // PhoneMenu function

//-------------------------------------------------------------------------------------------


Initialize();

if     ($HTTP_GET_VARS["menu"] == "email"     ) { EmailMenu();   }
elseif ($HTTP_GET_VARS["menu"] == "submenu"   ) { SubMenu();     }
elseif ($HTTP_GET_VARS["menu"] == "phone"     ) { PhoneMenu();   }
elseif ($HTTP_GET_VARS["menu"] == "dothecall" ) { DoTheCall();   }
else                                            { MainMenu();    }

Closure();

?>