print "Content-type: text/html\n\n";

# ---------------------------------------------------------------
#
# -----------------------------------------------------
# copyright 2000 by voxeo corporation. (see LGPL.txt)
# -----------------------------------------------------
#
# v1.0 Coded in Perl
# v1.0 Coded by Stephen J. Lewis
#
# This application is a simple "Telephone Directory" or contact list
# that uses a comma delimited file for contact information.  It allows
# the caller to dial into a number, and then place an outbound call
# to a phone number in the list or send a voice message as an email
# attachment to an email address in the list.  If either of those are
# missing, the caller can still use the other sub.  This application
# was also coded in ASP and PHP.
#
# This zip file for this application contains the following files:				 
#     contactlist.pl        <---- This file
#     contactlist.asp       <---- ASP version
#     contactlist.php       <---- PHP version
#     contactlist.ini       <---- Comma delimited configuration file.  See the
#                                 header in that file for more details.
#     LGPL.txt              <---- opensource license agreement
#     readme.txt            <---- Introductory text file
#
# Requires:
#     CGI.pm
#
# Several audio files are associated with this application:
#           greeting.wav
#           nocontactinfo.wav
#           onlyphone.wav
#           onlyemail.wav
#           bothphoneemail.wav
#           starkey.wav
#           returningtomainmenu.wav
#           emailmessage.wav
#           errormessage.wav
#           inifileerror.wav
#           messagesent.wav
#           holdmusic.wav
#           callfailure.wav
#
# All audio files are PCM format, 8-bit, mono, unsigned.
#
#----------------------------------------------------------------

#------------------------------------------------------------------------
# Change this variable to reflect your path
#
# $Filename = "c:/inetpub/wwwroot/jumpstart/contactlist/contactlist.ini";
# 
# Or use the code below to use a relative filepath
#------------------------------------------------------------------------

use FindBin;
$Filename = "$FindBin::Bin/contactlist.ini";

#-------------------------------------------------------------------------------------------

sub CleanUpString($TempString) {
  #-----------------------------------------------------------
  # We have to turn the %20 into a space from the command line
  #-----------------------------------------------------------  
  
  $Position = index($TempString, "%20");
	
  while ($Position > -1) {
	$TempString = substr($TempString, 0, $Position) . " " . substr($TempString, $Position + 3);
    $Position   = index($TempString, "%20");  	
  }  # if statement
  
  #---------------------------------------
  # Get rid of the whitespace at the front
  #---------------------------------------
  
  while (substr($TempString, 0, 1) eq " ") {
    $TempString = substr($TempString, 1); 
  }  # while loop
  
  #--------------------------------------
  # Get rid of the whitespace at the back
  #--------------------------------------
  
  while (substr($TempString, length($TempString), 1) eq " ") {
    $TempString = substr($TempString, 0, (length($TempString) -1));
  }  # while loop

  return $TempString;
}  # CleanUpString sub

#-------------------------------------------------------------------------------------------

sub Initialize {
  print "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"; 
  print "<callxml>";
  
  #------------------------------
  # Create a new cgi query module
  #------------------------------
  
  $query = new CGI;
  
  $CacheKiller = $query->param('cachekiller');
  $Menu        = $query->param('menu');  
  $Offset      = $query->param('offset');
  $Email       = $query->param('email');
  $Phone       = $query->param('phone');
  $Name        = $query->param('name');
  
  $TempString = $Name;
  $Name       = &CleanUpString($TempString);
  
  $TempString = $Email;
  $Email      = &CleanUpString($TempString);
  
  $TempString = $Phone;
  $Phone      = &CleanUpString($TempString);
  
  #------------------------------------------------------------------------
  # This annoying little variable is simply incremented and passed along to
  # avoid the AGS's caching.  This helps to ensure that a different URL
  # query string will be passed.
  #------------------------------------------------------------------------  

  if ($CacheKiller lt "1") { print "<assign var=\"cachekiller\" value=\"1\"/>";  }
  else                     { print "<assign var=\"cachekiller\" value=\"" . 
                                   ($CacheKiller + 1) . "\"/>";                  }
}  # Initialize sub

#-------------------------------------------------------------------------------------------

sub Closure {
  print "</callxml>";
}  # Closure sub

#-------------------------------------------------------------------------------------------

sub GetContactList {
  $Loopy = 0;
  
  open TempFileHandle, $Filename;

  #----------------------------
  # Read the file into an array
  #----------------------------

  @FileArray = <TempFileHandle>;
  
  #---------------
  # Close the file
  #---------------

  close TempFileHandle;

  foreach $Line (@FileArray) {
	 #---------------------------------------------------
	 # We only want lines that do not start with a # sign
	 # (which is a commented line), and lines that have
	 # commas in them.
	 #---------------------------------------------------
	 
	 $IsThereAComma = index($Line, ",");
	 $PoundSign     = index($Line, "#");
	 
	 if (($PoundSign != 0) && ( $IsThereAComma > -1 )) {
       @ContactInfo[$Loopy] = $Line;
       $Loopy++;
     }  # if statement
  }  # foreach loop

  if ($Loopy eq "0") { @ContactInfo[1] = "FILE NOT FOUND"; }
  
  #-----------------
  # Return our array
  #-----------------
  
  return @ContactInfo;
}  # GetContactList sub

#-------------------------------------------------------------------------------------------

sub GetTextMessage(@ContactList) {
  $Loopy = 0;
  
  #---------------------------------------------------
  # We want to use the offset variable because we only
  # read 10 contacts at a time.  We obviously loop no
  # more than 10 times as a result of that fact.
  #--------------------------------------------------- 
   
  while ((@ContactList[$Loopy + $Offset] ne "") && ($Loopy < 10)) {
    #-----------------------------------------------------------------------------
    # Here we are simply splitting the line into its component parts via the comma
	#-----------------------------------------------------------------------------
    
    @TempArray = split /,/, @ContactList[$Loopy + $Offset];
						
	#---------------------------------------------------------------------
	# If there is no name, then simply use "Unnamed Contact" when reading
	#---------------------------------------------------------------------	
	
    if (@TempArray[0] eq "") { @TempArray[0] = "Unnamed Contact"; }
	
	#----------------------------------------------
	# The text array stores five important strings:
	#   text[1] is the message string
	#   text[2] is the name string
	#   text[3] is the phone string
	#   text[4] is the email string
	#   text[5] is the termdigit string
	#----------------------------------------------
	
	@Text[1] = @Text[1] . "For " . @TempArray[0] . " press " . $Loopy . ",";
	@Text[2] = @Text[2] . @TempArray[0] . ",";
	@Text[3] = @Text[3] . @TempArray[1] . ",";
	@Text[4] = @Text[4] . @TempArray[2] . ",";
	@Text[5] = @Text[5] . $Loopy;
	$Loopy++;				
  }  # while statement
  
  #-----------------------------------------------------------
  # If we are in the middle of the contact list, then tack
  # on a * as termdigit as the "move to the previous page" key.
  #------------------------------------------------------------
  
  if ($Offset ne "0") { 
    @Text[5] = @Text[5] . "*"; 
	@Text[1] = @Text[1] . "To move back in your list, press the star key,";		
  }
  
  #----------------------------------------------------------
  # If next item in the contact list is NOT empty, then tack
  # on a # as termdigit as the "move to the next page" key.
  #----------------------------------------------------------  
  
  if ((@ContactList[$Loopy + $Offset]) ne "") {
    @Text[5] = @Text[5] . "#";
	@Text[1] = @Text[1] . "To move forward in your list, press the pound key,";			
  }  # if statement
  
  return @Text;
}  # GetTextMessage sub

#-------------------------------------------------------------------------------------------

sub MainMenu {
  @ContactList = &GetContactList;
  
  if (@ContactList[1] eq "FILE NOT FOUND") {
    print "<playaudio value=\"inifileerror.wav\"/>";
  }  # if statement
  else {
    print "<block label=\"MainMenu\" " .
                 "repeat=\"3\">";     
				 
    #------------------------------------------------------------
    # Because we only play 10 contacts at a time, we only want to
    # play the initial greeting file if we are calling into the
    # list for the first time.
    #------------------------------------------------------------
  
    if ($Offset eq "") { 
	  $Offset    = "0"; 
	  $FirstPass = "YES";
	}  # if statement
  
    if ($FirstPass eq "YES") {
      print "<playaudio value=\"greeting.wav\"/>";
    }  # if statement
  
    #---------------------------------------------------------------------------
    # Get the text message from the array of contacts (uses the offset variable)
    #---------------------------------------------------------------------------
  
    @Text = GetTextMessage(@ContactList);
			   
    print "<text termdigits=\"" . @Text[5] . "\">";
    print @Text[1];
    print "</text>";
  
    #-------------------------------------------------------
    # Now we want to loop through and add our event handlers
    #-------------------------------------------------------
  
    for ($Loopy = 0; $Loopy < length(@Text[5]); $Loopy++) {
      $TempString = @Text[5];
      print "<ontermdigit value=\"" . substr($TempString, $Loopy, 1) . "\">";
	
	  #---------------------------
	  # Move to previous set of 10
	  #---------------------------
	
	  $TempString = @Text[5];
	  if (substr($TempString, $Loopy, 1) eq "*") {
	    $TempOffset = $Offset - 10;
	  
	    if ($TempOffset < 0) { $TempOffset = 0; }
	  
	    print "<goto value=\"contactlist.pl?menu=mainmenu&amp;offset=" . $TempOffset . "\"/>";	  
	  }  # if statement
	
	  #-----------------------
	  # Move to next set of 10
	  #-----------------------
	
	  elsif (substr($TempString, $Loopy, 1) eq "#") {
	    $TempOffset = $Offset + 10;
	  
	    print "<goto value=\"contactlist.pl?menu=mainmenu&amp;offset=" . $TempOffset . "\"/>";		
	  }  # elsif statement
	
	  #--------------------
	  # Actual contact name
	  #--------------------
	
	  else {
	    #-----------------------------------------------
		# We want to extract from our individual strings
		# the email, telephone number, and name of the
		# contact according to our loop variable.
		#-----------------------------------------------
		
	    $TempEmail = @Text[4];
	    @TempArray = split /,/, $TempEmail;
	    $Email     = @TempArray[$Loopy];
	  
	    $TempPhone = @Text[3];
	    @TempArray = split /,/, $TempPhone;
	    $Phone     = @TempArray[$Loopy];
	  
	    $TempName = @Text[2];
	    @TempArray = split /,/, $TempName;
	    $Name      = @TempArray[$Loopy];
	  
	    print "<goto value=\"contactlist.pl?menu=submenu&amp;email=" . $Email . 
	              "&amp;phone="  . $Phone .
				  "&amp;name="   . $Name  .
	  			  "&amp;offset=" . $Offset . "\" " .
				  "submit=\"*\" method=\"get\"/>";
	  }  # else statement
	
	  print "</ontermdigit>";
    }  # for loop  
	
    print "<wait value=\"3s\" termdigits=\"" . @Text[5] . "\"/>";  
  
    print "</block>";	
  }  # else statement (ini file was found)
}  # MainMenu sub

#-------------------------------------------------------------------------------------------

sub SubMenu {
  print "<block label=\"SubMenu\" repeat=\"3\">";
  
  #--------------------------------------------------------------------
  # We need to play the appropriate audio file depending on the contact
  # information that is stored in the configuration file.
  #--------------------------------------------------------------------
  
  if (($Email eq "") && ($Phone eq "")) { 
    print "<playaudio value=\"nocontactinfo.wav\"/>";
	$TermDigits = ""; 
  }  # if statement
  elsif (($Email eq "") && ($Phone ne "")) { 
    print "<playaudio value=\"onlyphone.wav\" " . "termdigits=\"1*\"/>"; 
	$TermDigits = "1*";
  }    # elsif statement
  elsif (($Email ne "") && ($Phone eq "")) { 
    print "<playaudio value=\"onlyemail.wav\" " . "termdigits=\"2*\"/>"; 
	$TermDigits = "2*";
  }  # elsif statement
  else { 
    print "<playaudio value=\"bothphoneemail.wav\" " . "termdigits=\"12*\"/>"; 
	$TermDigits = "12*";
  }  # else statement
  
  #-----------------------
  # Read the person's name
  #-----------------------
  
  print "<text termdigits=\"" . $TermDigits . "\">" . $Name . "</text>";	
  
  #---------------------------------------------------------------------------
  # We do not need to play "Press the Star Key to return to the main menu"
  # if there is no contact information (because we are already returning there.
  #----------------------------------------------------------------------------
  
  if ($TermDigits ne "") {
    print "<playaudio value=\"starkey.wav\" termdigits=\"" . $TermDigits . "\"/>";
  }  # if statement
  
  #--------------------------------------------------------
  # We need to use the appropriate event handlers depending
  # on what contact information we have.
  #--------------------------------------------------------
  
  if ($Phone ne "") { 
	print "<ontermdigit value=\"1\">";
	print "<goto value=\"contactlist.pl?menu=phone&amp;phone=" . $Phone . 
	  			  "&amp;offset=" . $Offset . "\" " .
				  "submit=\"*\" method=\"get\"/>";
	print "</ontermdigit>";
  }  # if statement (Phone exists)
  
  if ($Email ne "") {
	print "<ontermdigit value=\"2\">";
	print "<goto value=\"contactlist.pl?menu=email&amp;email=" . $Email . 
	  			  "&amp;offset=" . $Offset . "\" " .
				  "submit=\"*\" method=\"get\"/>";	
	print "</ontermdigit>";		
  }  # if statement (Email exists)
  
  if (($Email ne "") || ($Phone ne "")) {
	print "<ontermdigit value=\"*\">";
	print "<goto value=\"contactlist.pl?menu=mainmenu" .
	  			  "&amp;offset=" . $Offset . "\" " .
				  "submit=\"*\" method=\"get\"/>";	
	print "</ontermdigit>";		  
  }  # if statement (Phone or Email exist)
  else {
	print "<playaudio value=\"returningtomainmenu.wav\"/>";
	print "<goto value=\"contactlist.pl?menu=mainmenu&amp;offset=" . $Offset . "\"/>";
  }  # else statement  
 
  print "<wait value=\"3s\" termdigits=\"" . @Text[5] . "\"/>";
  
  print "</block>";
  
  print "<goto value=\"contactlist.pl?menu=mainmenu&amp;offset=" . $Offset . "\"/>";  
}  # SubMenu sub

#-------------------------------------------------------------------------------------------

sub EmailMenu {
  print "<block>";
  print "<cleardigits/>";
  print "<playaudio value=\"EmailMessage.wav\" termdigits=\"*#\"/>";
  print "<recordaudio format=\"audio/wav\" " .
                     "value=\"mailto:" . $Email . "\" " .
					 "maxtime=\"180s\" " .
					 "maxsilence=\"20s\" " .
					 "cleardigits=\"TRUE\" " .
					 "beep=\"TRUE\" " .
					 "termdigits=\"*#\"/>";
					 
  print "<onerror>";
  print "<playaudio value=\"errormessage.wav\"/>";
  print "<goto value=\"contactlist.pl?menu=mainmenu&amp;offset=" . 
  			  $Offset . "\"/>";   
  print "</onerror>";
  
  print "<onmaxtime/>";
  print "<onmaxsilence/>";
  
  print "<ontermdigit value=\"*\">";
  print "<playaudio value=\"messagesent.wav\"/>";  
  print "<goto value=\"contactlist.pl?menu=mainmenu&amp;offset=" . 
  			  $Offset . "\"/>";   
  print "</ontermdigit>";
  
  print "<ontermdigit value=\"#\">";
  print "<playaudio value=\"messagesent.wav\"/>";  
  print "<goto value=\"contactlist.pl?menu=mainmenu&amp;offset=" . 
  			  $Offset . "\"/>";   
  print "</ontermdigit>";  

  print "</block>";
}  # EmailMenu sub

#-------------------------------------------------------------------------------------------

sub DoTheCall {
  #----------------------------------------------------
  # This sub does the actual outbound call, and
  # then sends back the results via callXML <sendevent>
  # actions.
  #----------------------------------------------------
  
  print "<block>";

  print "<call value=\"" . $Phone . "\" " .
       "maxtime=\"25s\"/>";
		
  #-----------------------------------------------------
  # Send back the appropriate event to the other session
  #-----------------------------------------------------
		
  print "<onanswer>";
  print "  <sendevent value=\"success\" session=\"\$parentsessionid;\"/>";
  print "  <wait value=\"Unlimited\"/>";			   
  print "</onanswer>";
  
  print "<oncallfailure>";
  print "  <sendevent value=\"busy\" session=\"\$parentsessionid;\"/>";
  print "</oncallfailure>";
  
  print "<onmaxtime>";
  print "  <sendevent value=\"timedout\" session=\"\$parentsessionid;\"/>";
  print "</onmaxtime>";
  
  print "<onerror>";
  print "  <sendevent value=\"error\" session=\"\$parentsessionid;\"/>";
  print "</onerror>";
  
  print "<onexternalevent value=\"kill\">";
  print "  <hangup/>";
  print "</onexternalevent>";  
  
  print "</block>";
}  # DoTheCall sub

#----------------------------------------------------------------------------------------


sub PhoneMenu {
  print "<block>";
	
  #-------------------------------------------------------
  # We need to assign a ParentSessionID variable so that
  # the session we call with the <run> command can send us
  # a callXML event back.  Without assigning that variable,
  # there is no way to remember/determine the parent 
  # session's ID.  Remember, \$ is the escape character for
  # $ in pl (as seen in the line directly below...)
  #--------------------------------------------------------
	 
  print "<assign var=\"parentsessionid\" value=\"\$session.id;\"/>";
	
  print "<run value=\"contactlist.pl?menu=dothecall" . 
        "&amp;phone=" . $Phone . 
		"&amp;offset=" . $Offset . "\" " .
	    "submit=\"*\" " .
	    "method=\"get\" " .
	    "var=\"ChildSession\"/>";
	   
  print "<block label=\"Hold Music\" " .
	          "repeat=\"3\">";
	   
  #-----------------------------------------------------------------
  # Play holdmusic while waiting for an event from the other session
  #-----------------------------------------------------------------
		   
  print "<playaudio value=\"holdmusic.wav\"/>";
	   
  #----------------------------------------------------
  # Conference the call when someone/something picks up
  #----------------------------------------------------
		
  print "<onexternalevent value=\"success\">";
  print "  <conference targetsessions=\"\$session.eventsenderID;\"/>";
  print "  <hangup/>";
  print "</onexternalevent>";
	 
  #-------------------------------------------
  # Inform the caller when the number was busy
  #-------------------------------------------
		
  print "<onexternalevent value=\"busy\">";
  print "  <playaudio value=\"callfailure.wav\"/>";					 
  print "  <sendevent value=\"kill\" session=\"\$ChildSession;\"/>";
  print "  <goto value=\"contactlist.pl?menu=mainmenu" . 
              "&amp;offset=" . $Offset . "\" submit=\"*\" method=\"get\"/>";
  print "</onexternalevent>";
	 
  #-------------------------------------------
  # Inform the caller when the number timedout
  #-------------------------------------------
	
  print "<onexternalevent value=\"timedout\">";
  print "  <playaudio value=\"callfailure.wav\"/>";					 
  print "  <sendevent value=\"kill\" session=\"\$ChildSession;\"/>";
  print "  <goto value=\"contactlist.pl?menu=mainmenu" . 
              "&amp;offset=" . $Offset . "\" submit=\"*\" method=\"get\"/>";
  print "</onexternalevent>";
	 
  #-----------------------------------------------
  # Inform the caller when there has been an error
  #-----------------------------------------------  
	 
  print "<onexternalevent value=\"error\">";
  print "  <playaudio value=\"callfailure.wav\"/>";					 
  print "  <sendevent value=\"kill\" session=\"\$ChildSession;\"/>";
  print "  <goto value=\"contactlist.pl?menu=mainmenu" . 
              "&amp;offset=" . $Offset . "\" submit=\"*\" method=\"get\"/>";
  print "</onexternalevent>";  
	
  print "</block>";
	 
  #-------------------------------------------------------
  # In theory, we shouldn't reach this point.  But just in
  # case, we might want to give the user a message.
  #-------------------------------------------------------
	 
  print "  <playaudio value=\"callfailure.wav\"/>";					 
  print "  <sendevent value=\"kill\" session=\"\$ChildSession;\"/>";
  print "  <goto value=\"contactlist.pl?menu=mainmenu" . 
              "&amp;offset=" . $Offset . "\" submit=\"*\" method=\"get\"/>";
  print "</block>";
}  # PhoneMenu sub

#-------------------------------------------------------------------------------------------

# ------------------------
# Initialize the perl libs
# ------------------------
  
use CGI;

&Initialize;

if    ($Menu eq "email"     ) { &EmailMenu;   }
elsif ($Menu eq "submenu"   ) { &SubMenu;     }
elsif ($Menu eq "phone"     ) { &PhoneMenu;   }
elsif ($Menu eq "dothecall" ) { &DoTheCall;   }
else                          { &MainMenu;    }

&Closure;
